(:~
 : A helper module that offers functionalities that are needed
 : while serializing the TEI encoding into XHTML.
 :
 : @author Michelle Weidling
 :
 :)

xquery version "3.1";

module namespace commons="http://bdn-edition.de/ns/commons";

declare namespace bdn="http://www.bdn-edition.de/ns/1.0";
declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace xhtml="http://www.w3.org/1999/xhtml";

import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";
import module namespace id-file="https://bdn-edition.de/ns/id-file" at "id-file.xqm";
(:import module namespace console="http://exist-db.org/xquery/console";:)

(:~
 : Creates an anchor as target for links within the edition.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $xml-id The ID of the element, mostly of a former tei:head
 : @return element(xhtml:span) An invisible xhtml:span as anchor point
 :)
declare function commons:make-text-anchor($xml-id as xs:string) as element(span)
{
  element xhtml:span
  {
    attribute class {"text-anchor"},
    attribute id {$xml-id}
  }
};


(:~
 : Looks up the author of the work a given file belongs to. For works that have
 : already been published, no look up is necessary, but since links should also
 : work for unpublished documents to enable correcting links, we have created
 : a map, whitelist.xml, that contains all necessary information.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $file The current file, e.g. "2228v"
 : @return The author of the work the file belongs to
 :)
declare function commons:get-author($file as xs:string*) as xs:string*
{
  let $whitelist := doc($config:app-root || "/resources/aux/whitelist.xml")
  let $first-char := substring($file, 1, 1)
  let $uri :=
    if (matches($first-char, "[0-9]")) then
      substring-before($file, ".")
    else
      ()
  return
    if ($uri) then
      $whitelist//bdn:author[matches(following-sibling::bdn:files//*, $uri)]/string()
    else
      substring-before($file, "-")
};


(:~
 : Creates a whitespace depending on the flag @break-before that is set while
 : serializing the intermediate format.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node The current node
 : @return A whitespace if the flag is set
 :)
declare function commons:make-whitespace-before($node as node()) as text()*
{
  if ($node/@break-before = "yes") then
    text {" "}
  else
    ()
};


(:~
 : Creates a whitespace depending on the flag @break-after that is set while
 : serializing the intermediate format.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node The current node
 : @return A whitespace if the flag is set
 :)
declare function commons:make-whitespace-after($node as node()) as text()*
{
  if ($node/@break-after = "yes") then
    if ($node/following::text()[1][starts-with(., ",") or starts-with(., ")")]) then
      ()
    else if ($node[self::tei:ptr]
    and $node/preceding::text()[1][ends-with(., "{") or ends-with(., "(")]) then
      ()
    else
      text {" "}
  else
    ()
};


(:~
 : Checks if the next text node of the main text begins with punctuation.
 : Edit for KMG: Also checks if the node is the last child-element of a tei:quote or tei:q without no text following.
 :
 : @author Michelle Weidling
 : @author Max-Ferdinand Zeterberg
 : @version 2.0
 : @param $node The current node
 : @param $mode The current view. In case of the single edition view, $mode
 : denotes the current printing
 : @return true() if next text node begins with punctuation
 :)
declare function commons:starts-next-text-with-punctuation($node as node(),
$mode as xs:string) as xs:boolean
{
  let $next-valid-text :=
    if ($mode = ("app", "digital")) then
      $node/following::text()[not(normalize-space(.) = "")
      (: this is important for cases where we have more than one rdg of a tei:app.
      without this query the next valid text node would be the text node of the
      following tei:rdg, whereas we want the next main text :)
      and not(./ancestor::tei:rdg[@type = ("v", "pp", "pt")])][1]
    else
      $node/following::text()[not(normalize-space(.) = "")
      (: in the single edition view, we don't mind text nodes that appear
      in tei:rdgs other than the current printing :)
      and not(./ancestor::tei:rdg[not(matches(./@wit, $mode))])][1]

  let $first-char := substring($next-valid-text, 1, 1)
  return
    if ((matches($first-char, "[–\?!\.,\)};“:]")) or ((not($node/following-sibling::text()[(normalize-space(.) = "")])) and ($node/parent::tei:q or $node/parent::tei:quote))) then
      true()
    else
      false()
};

(:~
 : Creates a relative URL to an object given in a tei:ref's target or tei:choice's
 : resp attribute.
 : In the BdN, a tei:ref denotes a reference to another chapter, paragraph, note,
 : etc. within the current BdN work. A tei:choice with an @corresp is part of an
 : authorial corrigendum.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node A tei:ref or tei:choice element
 : @return The complete relative URL to the referenced object, e.g.
 : "book.html?file=griesbach-1-2&edition=digital#gr_section_19"
 :)
declare function commons:get-link-url($node as element(*), $mode as xs:string)
as xs:string
{
  let $id :=
    if ($node[self::tei:ref]) then
      replace($node/@target, "#", "")
    (: $node[self::tei:choice] :)
    else
      replace($node/@corresp, "#", "")
  let $id-file := doc($config:aux || "id-file.xml")
  let $target-file := $id-file//*[./string() = $id]/parent::bdn:file/@uri/string()

  return "book.html?file=" || $target-file || "&amp;edition=" || $mode || "#" || $id
};

declare function commons:get-link-url-to-corresp-node($node as element(*), $filename as xs:string,
$mode as xs:string) as xs:string
{
    "book.html?file=" || $filename || "&amp;edition=" || $mode || "#" || $node/@xml:id
};
